import java.io.StringWriter;
import java.util.concurrent.CountDownLatch;
import javax.script.*;
import org.junit.Test;

import static junit.framework.TestCase.assertEquals;

public class ScriptEngineWriterTest {
    @Test
    public void should_not_share_writers() throws ScriptException, InterruptedException {
        final StringWriter firstWriter = new StringWriter();
        final StringWriter secondWriter = new StringWriter();

        final CountDownLatch countDownLatch = new CountDownLatch(2);
        new Thread() {
            @Override
            public void run() {
                ScriptContext firstContext = new SimpleScriptContext();
                firstContext.setWriter(firstWriter);

                ScriptEngineManager firstManager = new ScriptEngineManager();
                final ScriptEngine firstEngine = firstManager.getEngineByName("python");
                firstEngine.setContext(firstContext);

                try {
                    firstEngine.eval("print 'first'");
                } catch (ScriptException e) {
                    e.printStackTrace();
                } finally {
                    countDownLatch.countDown();
                }
            }
        }.start();
        new Thread() {
            @Override
            public void run() {
                ScriptContext secondContext = new SimpleScriptContext();
                secondContext.setWriter(secondWriter);

                ScriptEngineManager secondManager = new ScriptEngineManager();
                final ScriptEngine secondEngine = secondManager.getEngineByName("python");
                secondEngine.setContext(secondContext);

                try {
                    secondEngine.eval("print 'second'");
                } catch (ScriptException e) {
                    e.printStackTrace();
                } finally {
                    countDownLatch.countDown();
                }
            }
        }.start();
        countDownLatch.await();

        assertEquals(firstWriter.toString(), "first\n");
        assertEquals(secondWriter.toString(), "second\n");
    }
}
